%% INITIALIZATION

% SCRIPT DEVELOPED BY:
%   Benjamin Brewer
%   Leonidas Bantis (leobantis.net)

% INSTRUCTIONS FOR USE:
%   1. Ensure that your dataset is of the same format as the attached
%      sample dataset. This script only accommodates situations in which
%      h = 2 (two ambiguously ordered groups).
%   2. Make sure that you have put all necessary files MATLAB's Current
%      Folder.
%   3. Set values for SEED, BOOTS, N1, N2, N3, N4, and PATH, and FLAG.
%   4. Click anywhere in this chunk, and then click "Run Section."
%   5. Click line 33, and then click "Run Section."
%   6. Progress will output to the screen as the run progresses. Note that
%      the kernel method is particularly slow and may take a long time to
%      run with a large number of bootstraps. At the end of the run, plots
%      will automatically pop up on the screen, and other results of
%      interest will populate in the Command Window.

clc; close all; clear all;

% RANDOMIZATION SEED.
SEED = 256;

% NUMBER OF BOOTSTRAPPED ITERATIONS.
BOOTS = 10;

% SAMPLE SIZE OF EACH CLASS.
N1 = 97;
N2 = 100;
N3 = 40;
N4 = 40;

% NAME OF .CSV FILE ONLY. NEEDS TO BE IN CURRENT MATLAB FOLDER.
PATH = 'example';

% DECIDE WHICH ANALYSES YOU WANT TO RUN:
%   0 = NEITHER (SURFACE PLOT & VUS ONLY)
%   1 = BOX-COX ONLY
%   2 = KERNEL ONLY
%   3 = BOTH
FLAG = 3;

%% RUN THIS CHUNK FOR RESULTS

n = [N1 N2 N3 N4];

% BOX-COX

if (FLAG == 1) | (FLAG == 3)

    dataFrame = readtable(PATH);
    dataFrame = table2array(dataFrame);

    N = n;

    dataFrame = dataFrame;

    X1 = dataFrame(:, 1); X1 = X1(1:N(1));
    X2 = dataFrame(:, 2); X2 = X2(1:N(2));
    X3 = dataFrame(:, 3); X3 = X3(1:N(3));
    X4 = dataFrame(:, 4); X4 = X4(1:N(4));

    ll = @(l) -1*lambda_profile_LL_4grp(X1, X2, X3, X4, l);
    lhat = fminsearch(ll, 1);

    if (lhat < 0.001)
        lhat = 0.001;
    end

    X1_bc = ((X1.^lhat) - 1) / lhat;
    X2_bc = ((X2.^lhat) - 1) / lhat;
    X3_bc = ((X3.^lhat) - 1) / lhat;
    X4_bc = ((X4.^lhat) - 1) / lhat;

    [h1, p1_pre] = lillietest(X1); [h1, p1] = lillietest(X1_bc);
    [h2, p2_pre] = lillietest(X2); [h2, p2] = lillietest(X2_bc);
    [h3, p3_pre] = lillietest(X3); [h3, p3] = lillietest(X3_bc);
    [h4, p4_pre] = lillietest(X4); [h4, p4] = lillietest(X4_bc);

    clc;

    mu = [mean(X1_bc) mean(X2_bc) mean(X3_bc) mean(X4_bc)];
    sd = [std(X1_bc) std(X2_bc) std(X3_bc) std(X4_bc)];

    yi = @(c) -1*(YI_N2(c(1), c(2), mu, sd, 0));
    [chat_yi, J1] = fminsearch(@(c) yi([c(1) c(2)]), [mean([mu(1) mu(2)]) mean([mu(2) mu(3)])]);

    nwct = @(c) NWCT_N(c(1), c(2), mu, sd);
    [chat_nwct, d] = fminsearch(@(c) nwct([c(1) c(2)]), [mean([mu(1) mu(2)]) mean([mu(2) mu(3)])]);

    rng(SEED)

    for b = 1:BOOTS

        if (mod(b, 10) == 0)
            clc;
            fprintf("BOX-COX\nITERATION %i OF %i \n\n", b, BOOTS)
        end

        boots1 = randsample(1:N(1), N(1), true);
        boots2 = randsample(1:N(2), N(2), true);
        boots3 = randsample(1:N(3), N(3), true);
        boots4 = randsample(1:N(4), N(4), true);

        X1_boot = X1(boots1);
        X2_boot = X2(boots2);
        X3_boot = X3(boots3);
        X4_boot = X4(boots4);

        ll_boot = @(l) -1*lambda_profile_LL_4grp(X1_boot, X2_boot, X3_boot, X4_boot, l);
        [lhat_boot, TRASH, ef] = fminsearch(ll_boot, 1, optimset('Display','off'));

        if (lhat_boot < 0.001)
            lhat_boot = 0.001;
        end

        X1_bc = ((X1_boot.^lhat_boot) - 1) / lhat_boot;
        X2_bc = ((X2_boot.^lhat_boot) - 1) / lhat_boot;
        X3_bc = ((X3_boot.^lhat_boot) - 1) / lhat_boot;
        X4_bc = ((X4_boot.^lhat_boot) - 1) / lhat_boot;

        mu = [mean(X1_bc) mean(X2_bc) mean(X3_bc) mean(X4_bc)];
        sd = [std(X1_bc) std(X2_bc) std(X3_bc) std(X4_bc)];

        guess1 = ((chat_yi(1)^lhat_boot) - 1) / lhat_boot;
        guess2 = ((chat_yi(2)^lhat_boot) - 1) / lhat_boot;

        if (ef == 1)

            yi = @(c) -1*(YI_N2(c(1), c(2), mu, sd, 0));
            [chat_boot_yi, J1] = fminsearch(@(c) yi([c(1) c(2)]), [guess1 guess2]);
            lboot_yi(b, :) = ((lhat_boot.*chat_boot_yi) + 1).^(1/lhat_boot);

            nwct = @(c) NWCT_N(c(1), c(2), mu, sd);
            [chat_boot_nwct, d] = fminsearch(@(c) nwct([c(1) c(2)]), [guess1 guess2]);
            lboot_nwct(b, :) = ((lhat_boot.*chat_boot_nwct) + 1).^(1/lhat_boot);

            exitflags(b) = ((mu(1) < mu(2)) & (mu(2) < mu(3)) & (mu(2) < mu(4)))*1;

        else

            exitflags(b) = 0;

        end

    end

    keepBoot = find(exitflags == 1);

    chat_yi = ((lhat.*chat_yi) + 1).^(1/lhat);
    chat_nwct = ((lhat.*chat_nwct) + 1).^(1/lhat);

    var_c1_yi = var(lboot_yi(keepBoot, 1));
    var_c1_nwct = var(lboot_nwct(keepBoot, 1));

    var_c2_yi = var(lboot_yi(keepBoot, 2));
    var_c2_nwct = var(lboot_nwct(keepBoot, 2));

    % CIs

    CI_c1_yi = [chat_yi(1) - 1.96*sqrt(var_c1_yi) chat_yi(1) + 1.96*sqrt(var_c1_yi)];
    CI_c1_nwct = [chat_nwct(1) - 1.96*sqrt(var_c1_nwct) chat_nwct(1) + 1.96*sqrt(var_c1_nwct)];

    CI_c2_yi = [chat_yi(2) - 1.96*sqrt(var_c2_yi) chat_yi(2) + 1.96*sqrt(var_c2_yi)];
    CI_c2_nwct = [chat_nwct(2) - 1.96*sqrt(var_c2_nwct) chat_nwct(2) + 1.96*sqrt(var_c2_nwct)];

    clc;

    % PRINT RESULTS

    BOXCOX_TRANSFORM_SUCCESSFUL = ["X1:" p1>0.05;
                                   "X2:" p2>0.05;
                                   "X3:" p3>0.05;
                                   "X4:" p4>0.05];

    BOXCOX_YI_RESULTS  = ["chat_1:" chat_yi(1) "chat_2:" chat_yi(2);
                          "var_chat_1:" var_c1_yi "var_chat_2:" var_c2_yi;
                          "c1_lcl:" CI_c1_yi(1) "c1_ucl:" CI_c1_yi(2);
                          "c2_lcl:" CI_c2_yi(1) "c2_ucl:" CI_c2_yi(2)];

    BOXCOX_NWCT_RESULTS = ["chat_1:" chat_nwct(1) "chat_2:" chat_nwct(2);
                           "var_chat_1:" var_c1_nwct "var_chat_2:" var_c2_nwct;
                           "c1_lcl:" CI_c1_nwct(1) "c1_ucl:" CI_c1_nwct(2);
                           "c2_lcl:" CI_c2_nwct(1) "c2_ucl:" CI_c2_nwct(2)];

end

% KERNELS

if (FLAG == 2) | (FLAG == 3)

    dataFrame = readtable(PATH);
    dataFrame = table2array(dataFrame);

    ncol = size(dataFrame);
    ncol = ncol(2);

    for l = 1:ncol
        inVec = dataFrame(:, l);
        inVec(inVec == 0) = NaN;
        dataFrame(:, l) = inVec;
    end

     muEst = nanmean(dataFrame);
     sdEst = nanstd(dataFrame);

     yiSim = @(c) -1*(YI_KERNEL(dataFrame, c(1), c(2), 0));
     nwctSim = @(c) NWCT_KERNEL(dataFrame, c(1), c(2));

     [chat_yi] = fminsearch(@(c) yiSim([c(1) c(2)]), [2.75 3.75]);
     [chat_nwct] = fminsearch(@(c) nwctSim([c(1) c(2)]), [2.75 3.75]);

     rng(SEED)

    for b = 1:BOOTS
        
        if (mod(b, 10) == 0)
            clc;
            fprintf("KERNEL\nITERATION %i OF %i \n\n", b, BOOTS)
        end

        bootFrame = zeros(max(n), ncol);
        for l = 1:ncol
            boots = randsample(1:n(l), n(l), true);
            inVec = repelem(0, max(n));
            inVec(1:n(l)) = dataFrame(boots, l);
            inVec(inVec == 0) = NaN;
            bootFrame(:, l) = inVec;
        end

        muEst_boot = nanmean(bootFrame);
        sdEst_boot = nanstd(bootFrame);

        yiSim_boot = @(c) -1*(YI_KERNEL(bootFrame, c(1), c(2), 0));
        nwctSim_boot = @(c) NWCT_KERNEL(bootFrame, c(1), c(2));

        [chat_yi_boot] = fminsearch(@(c) yiSim_boot([c(1) c(2)]), [mean([muEst_boot(1) muEst_boot(2)]) mean([muEst_boot(2) muEst_boot(3)])]);
        [chat_nwct_boot] = fminsearch(@(c) nwctSim_boot([c(1) c(2)]), [mean([muEst_boot(1) muEst_boot(2)]) mean([muEst_boot(2) muEst_boot(3)])]);

        lboot_yi(b, :) = chat_yi_boot;
        lboot_nwct(b, :) = chat_nwct_boot;
        
        exitflags(b) = ((muEst_boot(1) < muEst_boot(2)) & ...
                        (muEst_boot(2) < muEst_boot(3)) & ...
                        (muEst_boot(2) < muEst_boot(4)))*1;

    end

   keepBoot = find(exitflags == 1);
    
    var_c1_yi = var(lboot_yi(keepBoot, 1)); var_c2_yi = var(lboot_yi(keepBoot, 2));
    var_c1_nwct = var(lboot_nwct(keepBoot, 1)); var_c2_nwct = var(lboot_nwct(keepBoot, 2));

    CI_c1_yi = [chat_yi(1) - 1.96*sqrt(var_c1_yi) chat_yi(1) + 1.96*sqrt(var_c1_yi)];
    CI_c1_nwct = [chat_nwct(1) - 1.96*sqrt(var_c1_nwct) chat_nwct(1) + 1.96*sqrt(var_c1_nwct)];

    CI_c2_yi = [chat_yi(2) - 1.96*sqrt(var_c2_yi) chat_yi(2) + 1.96*sqrt(var_c2_yi)];
    CI_c2_nwct = [chat_nwct(2) - 1.96*sqrt(var_c2_nwct) chat_nwct(2) + 1.96*sqrt(var_c2_nwct)];

    % STORE RESULTS

    KERNEL_YI_RESULTS = ["chat_1:" chat_yi(1) "chat_2:" chat_yi(2);
                          "var_chat_1:" var_c1_yi "var_chat_2:" var_c2_yi;
                          "c1_lcl:" CI_c1_yi(1) "c1_ucl:" CI_c1_yi(2);
                          "c2_lcl:" CI_c2_yi(1) "c2_ucl:" CI_c2_yi(2)];

    KERNEL_NWCT_RESULTS = ["chat_1:" chat_nwct(1) "chat_2:" chat_nwct(2);
                           "var_chat_1:" var_c1_nwct "var_chat_2:" var_c2_nwct;
                           "c1_lcl:" CI_c1_nwct(1) "c1_ucl:" CI_c1_nwct(2);
                           "c2_lcl:" CI_c2_nwct(1) "c2_ucl:" CI_c2_nwct(2)];
             
end
        
% RESULTS

% PLOT OF SURFACE

dataFrame = readtable(PATH);
dataFrame = table2array(dataFrame);

ncol = size(dataFrame);
ncol = ncol(2);

 for l = 1:ncol
     inVec = dataFrame(:, l);
     inVec(inVec == 0) = NaN;
     dataFrame(:, l) = inVec;
 end

mu = nanmean(dataFrame);
sd = nanstd(dataFrame);

VUS = TROC_N_PLOT(mu, sd)

% PRINT ANY ANALYSES

% PRINT BOX-COX

if (FLAG == 1) | (FLAG == 3)
    
    BOXCOX_TRANSFORM_SUCCESSFUL
    BOXCOX_YI_RESULTS
    BOXCOX_NWCT_RESULTS

    figure;
    boxplot(dataFrame, 'Color', 'k', 'Symbol', '+k', 'Labels', ["X1" "X2" "X3" "X4"])
    ylabel('Value')
    xlabel('Group')
    hold on;
    plot([0 5], [str2num(BOXCOX_YI_RESULTS(1, 2)) str2num(BOXCOX_YI_RESULTS(1, 2))], '--b')
    hold on;
    plot([0 5], [str2num(BOXCOX_NWCT_RESULTS(1, 2)) str2num(BOXCOX_NWCT_RESULTS(1, 2))], '--r')
    hold on;
    plot([0 5], [str2num(BOXCOX_YI_RESULTS(1, 4)) str2num(BOXCOX_YI_RESULTS(1, 4))], '--b')
    hold on;
    plot([0 5], [str2num(BOXCOX_NWCT_RESULTS(1, 4)) str2num(BOXCOX_NWCT_RESULTS(1, 4))], '--r')
    title('BOX-COX: YI (blue) vs. Euclidean (red)')

end

if (FLAG == 2) | (FLAG == 3)

    KERNEL_YI_RESULTS
    KERNEL_NWCT_RESULTS

    figure;
    boxplot(dataFrame, 'Color', 'k', 'Symbol', '+k', 'Labels', ["X1" "X2" "X3" "X4"])
    ylabel('Value')
    xlabel('Group')
    hold on;
    plot([0 5], [str2num(KERNEL_YI_RESULTS(1, 2)) str2num(KERNEL_YI_RESULTS(1, 2))], '--b')
    hold on;
    plot([0 5], [str2num(KERNEL_NWCT_RESULTS(1, 2)) str2num(KERNEL_NWCT_RESULTS(1, 2))], '--r')
    hold on;
    plot([0 5], [str2num(KERNEL_YI_RESULTS(1, 4)) str2num(KERNEL_YI_RESULTS(1, 4))], '--b')
    hold on;
    plot([0 5], [str2num(KERNEL_NWCT_RESULTS(1, 4)) str2num(KERNEL_NWCT_RESULTS(1, 4))], '--r')
    title('KERNEL: YI (blue) vs. Euclidean (red)')
    
end
